package com.ruoyi.zhiyuanzhe.controller;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.zhiyuanzhe.domain.BPersonTags;
import com.ruoyi.zhiyuanzhe.service.IBPersonTagsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * 人标签Controller
 *
 * @author ruoyi
 * @date 2023-09-18
 */
@Api(tags = "人标签")
@RestController
@RequestMapping("/zhiyuanzhe/tags")
@Transactional(rollbackFor = Exception.class)
public class BPersonTagsController extends BaseController {

    @Resource
    private IBPersonTagsService bPersonTagsService;

    /**
     * 查询人标签列表
     */
    @ApiOperation(value = "查询人标签列表", response = BPersonTags.class)
    @GetMapping("/list")
    public TableDataInfo list(BPersonTags bPersonTags) {
        startPage();
        // 获得colors
        String[] ages = new String[0];
        if (bPersonTags.getAgeRange() != null && !bPersonTags.getAgeRange().isEmpty()) {
            ages = Convert.toStrArray(bPersonTags.getAgeRange());
        }
        bPersonTags.setAges(ages);
        List<BPersonTags> list = bPersonTagsService.selectBPersonTagsList(bPersonTags);
        return getDataTable(list);
    }


    /**
     * 获取人标签详细信息
     */
    @ApiOperation(value = "获取人标签详细信息", response = BPersonTags.class)
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bPersonTagsService.selectBPersonTagsById(id));
    }

    /**
     * 新增人标签
     */
    @ApiOperation(value = "新增人标签")
    @PostMapping
    public AjaxResult add(@RequestBody BPersonTags bPersonTags) {
        return toAjax(bPersonTagsService.insertBPersonTags(bPersonTags));
    }

    /**
     * 修改人标签
     */
    @ApiOperation(value = "修改人标签")
    @PutMapping
    public AjaxResult edit(@RequestBody BPersonTags bPersonTags) {
        return toAjax(bPersonTagsService.updateBPersonTags(bPersonTags));
    }

    /**
     * 删除人标签
     */
    @ApiOperation(value = "删除人标签")
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(bPersonTagsService.deleteBPersonTagsByIds(ids));
    }
}
