package com.ruoyi.zhiyuanzhe.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.zhiyuanzhe.domain.BActivityPoints;
import com.ruoyi.zhiyuanzhe.domain.BDistributionRecords;
import com.ruoyi.zhiyuanzhe.domain.request.BActivityPointsRequest;
import com.ruoyi.zhiyuanzhe.mapper.BActivityPointsMapper;
import com.ruoyi.zhiyuanzhe.service.IBActivityPointsService;
import com.ruoyi.zhiyuanzhe.service.IBDistributionRecordsService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;


/**
 * 活动积分Service业务层处理
 *
 * @author ruoyi
 * @date 2023-09-19
 */
@Service
public class BActivityPointsServiceImpl implements IBActivityPointsService {

    @Resource
    private BActivityPointsMapper bActivityPointsMapper;

    @Resource
    private ISysUserService sysUserService;

    @Resource
    private IBDistributionRecordsService distributionRecordsService;

    /**
     * 查询活动积分
     *
     * @param id 活动积分主键
     * @return 活动积分
     */
    @Override
    public BActivityPoints selectBActivityPointsById(Long id) {
        return bActivityPointsMapper.selectBActivityPointsById(id);
    }

    /**
     * 查询活动积分列表
     *
     * @param bActivityPoints 活动积分
     * @return 活动积分
     */
    @Override
    public List<BActivityPoints> selectBActivityPointsList(BActivityPoints bActivityPoints) {
        return bActivityPointsMapper.selectBActivityPointsList(bActivityPoints);
    }

    /**
     * 新增活动积分
     *
     * @param bActivityPoints 活动积分
     * @return 结果
     */
    @Override
    public int insertBActivityPoints(BActivityPoints bActivityPoints) {
        bActivityPoints.setCreateTime(DateUtils.getNowDate());
        return bActivityPointsMapper.insertBActivityPoints(bActivityPoints);
    }

    /**
     * 修改活动积分
     *
     * @param bActivityPoints 活动积分
     * @return 结果
     */
    @Override
    public int updateBActivityPoints(BActivityPoints bActivityPoints) {
        bActivityPoints.setUpdateTime(DateUtils.getNowDate());
        return bActivityPointsMapper.updateBActivityPoints(bActivityPoints);
    }

    /**
     * 批量删除活动积分
     *
     * @param ids 需要删除的活动积分主键
     * @return 结果
     */
    @Override
    public int deleteBActivityPointsByIds(Long[] ids) {
        return bActivityPointsMapper.deleteBActivityPointsByIds(ids);
    }

    /**
     * 删除活动积分信息
     *
     * @param id 活动积分主键
     * @return 结果
     */
    @Override
    public int deleteBActivityPointsById(Long id) {
        return bActivityPointsMapper.deleteBActivityPointsById(id);
    }

    /**
     * 发放一次性证书
     *
     * @param req 发放一次性证书请求类
     */
    @Override
    public void issue(BActivityPointsRequest req) {
        BActivityPoints entity = bActivityPointsMapper.selectBActivityPointsById(req.getId());
        if (BeanUtil.isEmpty(entity)) {
            throw new ServiceException("所选数据不存在");
        }
        // 获取用户地址
        SysUser sysUser = sysUserService.selectUserById(entity.getUId());
        // 创建证书发放记录
        BDistributionRecords records = new BDistributionRecords();
        if (StrUtil.isEmpty(sysUser.getAddress())) {
            throw new ServiceException("用户地址信息为空,不能发放");
        }
        records.setUId(SecurityUtils.getUserId());
        records.setAddress(sysUser.getAddress());
        records.setActivityId(req.getId());
        records.setCreType(1);
        records.setType(req.getType());
        records.setStatus(1);
        records.setCourierCompanies(req.getCourierCompanies());
        records.setTrackingNumber(req.getTrackingNumber());
        // 保存发放记录
        distributionRecordsService.insertBDistributionRecords(records);
    }

    /**
     * 获取用户所有积分
     *
     * @param userId 用户id
     * @return 积分
     */
    @Override
    public Integer findPointsByUserId(Long userId) {
        return bActivityPointsMapper.findPointsByUserId(userId);
    }
}

