package com.ruoyi.zhiyuanzhe.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.zhiyuanzhe.domain.BActivity;
import com.ruoyi.zhiyuanzhe.domain.BActivityUser;
import com.ruoyi.zhiyuanzhe.domain.BPersonTags;
import com.ruoyi.zhiyuanzhe.domain.request.ActivityInviteRequest;
import com.ruoyi.zhiyuanzhe.domain.request.EventRecommendationsRequest;
import com.ruoyi.zhiyuanzhe.domain.response.ActivityResponse;
import com.ruoyi.zhiyuanzhe.mapper.BActivityMapper;
import com.ruoyi.zhiyuanzhe.service.ActivityUserService;
import com.ruoyi.zhiyuanzhe.service.IBActivityService;
import com.ruoyi.zhiyuanzhe.service.IBPersonTagsService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 活动管理Service业务层处理
 *
 * @author ruoyi
 * @date 2023-09-18
 */
@Service("bActivityService")
public class BActivityServiceImpl implements IBActivityService {
    @Resource
    private BActivityMapper bActivityMapper;

    @Resource
    private ISysUserService userService;

    @Resource
    private IBPersonTagsService personTagsService;

    @Resource
    private ActivityUserService activityUserService;

    /**
     * 查询活动管理
     *
     * @param id 活动管理主键
     * @return 活动管理
     */
    @Override
    public ActivityResponse selectBActivityById(Long id) {
        // 查询活动数据
        BActivity activity = bActivityMapper.selectBActivityById(id);
        ActivityResponse res = BeanUtil.copyProperties(activity, ActivityResponse.class);
        // 查询已报名的人员信息
        List<BPersonTags> enrolledUsers = activityUserService.findUserByStatusAndActivityId(2, activity.getId());
        // 查询未报名人员信息或邀请的人员信息
        List<BPersonTags> unregisteredUsers = activityUserService.findUserByStatusAndActivityId(5, activity.getId());
        res.setEnrolledUsers(enrolledUsers);
        res.setUnregisteredUsers(unregisteredUsers);
        return res;
    }

    /**
     * 查询活动管理列表
     *
     * @param bActivity 活动管理
     * @return 活动管理
     */
    @Override
    public List<BActivity> selectBActivityList(BActivity bActivity) {

        return bActivityMapper.selectBActivityList(bActivity);
    }

    /**
     * 新增活动管理
     *
     * @param bActivity 活动管理
     */
    @Override
    public void insertBActivity(BActivity bActivity) {
        bActivity.setCreateTime(DateUtils.getNowDate());
        bActivity.setStatus(1);
        bActivityMapper.insertBActivity(bActivity);
        int score = getScore(bActivity);
        // 获取匹配到的人员
        List<BActivityUser> list = personTagsService.findByTags(bActivity);
        list.forEach(x -> {
            x.setStatus(0);
            if (score != 0) {
                BigDecimal mul = NumberUtil.mul(NumberUtil.div(x.getDegreeOfMatch(), score, 0), 100);
                x.setDegreeOfMatch(mul);
            }
            x.setActivityId(bActivity.getId());
        });
        List<BActivityUser> collect = list.stream().filter(x -> x.getDegreeOfMatch().compareTo(BigDecimal.ONE) > 0)
                .collect(Collectors.toList());
        activityUserService.addList(collect);
    }

    /**
     * 获取活动的所有匹配标签项
     *
     * @param bActivity 活动信息
     * @return 匹配的标签项个数
     */
    private int getScore(BActivity bActivity) {
        int score = 0;
        if (StrUtil.isNotEmpty(bActivity.getAgeRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getHousingRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getEducationRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getInterestRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getPoliticalRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getSexRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getNationalityRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getProfessionalRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getIndustryRange())) {
            score++;
        }
        if (StrUtil.isNotEmpty(bActivity.getSchoolRange())) {
            score++;
        }
        return score;
    }

    /**
     * 修改活动管理
     *
     * @param bActivity 活动管理
     */
    @Override
    public void updateBActivity(BActivity bActivity) {
        bActivityMapper.updateBActivity(bActivity);
        bActivity.setUpdateTime(DateUtils.getNowDate());
        // 删除所有人员
        activityUserService.delByActivityId(bActivity.getId());
        int score = getScore(bActivity);
        // 获取匹配到的人员
        List<BActivityUser> list = personTagsService.findByTags(bActivity);
        list.forEach(x -> {
            x.setStatus(0);
            if (score != 0) {
                BigDecimal mul = NumberUtil.mul(NumberUtil.div(x.getDegreeOfMatch(), score, 0), 100);
                x.setDegreeOfMatch(mul);
            }
            x.setActivityId(bActivity.getId());
        });
        List<BActivityUser> collect = list.stream().filter(x -> x.getDegreeOfMatch().compareTo(BigDecimal.ONE) > 0)
                .collect(Collectors.toList());
        activityUserService.addList(collect);
    }

    /**
     * 批量删除活动管理
     *
     * @param ids 需要删除的活动管理主键
     * @return 结果
     */
    @Override
    public int deleteBActivityByIds(Long[] ids) {
        int i = bActivityMapper.deleteBActivityByIds(ids);
        for (Long id : ids) {
            // 删除所有人员
            activityUserService.delByActivityId(id);
        }
        return i;
    }

    /**
     * 删除活动管理信息
     *
     * @param id 活动管理主键
     * @return 结果
     */
    @Override
    public int deleteBActivityById(Long id) {
        int i = bActivityMapper.deleteBActivityById(id);
        activityUserService.delByActivityId(id);
        return i;
    }

    /**
     * 邀请人员
     *
     * @param req 活动id和用户id
     */
    @Override
    public void invite(ActivityInviteRequest req) {
        activityUserService.invite(req);
    }

    /**
     * 活动推荐
     *
     * @param req 活动搜索条件
     * @return 活动信息
     */
    @Override
    public List<BActivity> eventRecommendations(EventRecommendationsRequest req) {
        return activityUserService.eventRecommendations(req);
    }

    /**
     * 报名
     *
     * @param req 活动id和用户id
     */
    @Override
    public void enroll(ActivityInviteRequest req) {
        activityUserService.enroll(req);
    }

    /**
     * 活动邀请列表
     *
     * @param req 活动搜索条件
     * @return 活动信息
     */
    @Override
    public List<BActivity> eventInvitations(EventRecommendationsRequest req) {
        return activityUserService.eventInvitations(req);
    }
}
